@@ -20,22 +20,40 @@ logger = logging.getLogger('pdf2doi')
 
 ######## Beginning first part, low-level functions ######## 
 
-#The list doi_regexp contains several regular expressions used to identify a DOI in a string. They are ordered from stricter to less and less strict
+doi_pattern = '\A10\.\d{4,9}/[-._;()/:A-Z0-9]+$' #This regex for the DOI is taken from https://www.crossref.org/blog/dois-and-matching-regular-expressions/
+                                                #and it is used to validate a given DOI.
+
+arxiv2007_pattern = '\A(\d{4}\.\d+)(?:v\d+)?$' #This is a regex for arxiv identifiers (in use after 2007) and it is used to validate a given arxiv ID.
+                                                                            
+
+#The list doi_regexp contains several regular expressions used to identify a DOI in a string. They are (roughly) ordered from stricter to less and less strict.
 doi_regexp = ['doi[\s\.\:]{0,2}(10\.\d{4}[\d\:\.\-\/a-z]+)(?:[\s\n\"<]|$)', # version 0 looks for something like "DOI : 10.xxxxS[end characters] where xxxx=4 digits, S=combination of characters, digits, ., :, -, and / of any length
                                                                             # [end characters] is either a space, newline, " , < or the end of the string. The initial part could be either "DOI : ", "DOI", "DOI:", "DOI.:", ""DOI:." 
                                                                             # and with possible spaces or lower cases.
               '(10\.\d{4}[\d\:\.\-\/a-z]+)(?:[\s\n\"<]|$)',                 # in version 1 the requirement of having "DOI : " in the beginning is removed
-              '(10\.\d{4}[\:\.\-\/a-z]+[\:\.\-\d]+)(?:[\s\na-z\"<]|$)']     # version 2 is useful for cases in which, in plain texts, the DOI is not followed by a space, newline or special characters,
-                                                                            #but is instead followed by other letters. In this case we can still isolate the DOI if we assume that the DOI always ends up with numbers
+              '(10\.\d{4}[\:\.\-\/a-z]+[\:\.\-\d]+)(?:[\s\na-z\"<]|$)',     # version 2 is useful for cases in which, in plain texts, the DOI is not followed by a space, newline or special characters,
+                                                                            #but is instead followed by other letters. In this case we can still isolate the DOI if we assume that the DOI always ends with numbers
+              'http[s]?://doi.org/(10\.\d{4,9}/[-._;()/:A-Z0-9]+)(?:[\s\n\"<]|$)', # version 3 is useful when the DOI can be found in a google result as an URL of the form https://doi.org/[DOI]
+                                                                            #The regex for [DOI] is 10\.\d{4,9}/[-._;()/:A-Z0-9]+ (taken from here https://www.crossref.org/blog/dois-and-matching-regular-expressions/)
+                                                                            #and it must be followed by a valid ending character: either a speace, a new line, a ", a <, or end of string.
+              '\A(10\.\d{4,9}/[-._;()/:A-Z0-9]+)$']                         # version 4 is like version 3, but without the requirement of the url https://doi.org/ in front of it.
+                                                                            #However, it requires that the string contains ONLY the doi and nothing else. This is useful for when the DOI is stored in metadata
 
-#Similarly, arxiv_regexp is a list of regular expressions used to identify an arXiv identifier in a string. They are ordered from stricter to less and less strict. Moreover,
+             
+#Similarly, arxiv_regexp is a list of regular expressions used to identify an arXiv identifier in a string. They are (roughly) ordered from stricter to less and less strict. Moreover,
 #the regexp corresponding to older arXiv notations have less priority.
 #NOTE: currently only one regexp is implemented for arxiv. 
 arxiv_regexp = ['arxiv[\s]*\:[\s]*(\d{4}\.\d+)(?:v\d+)?(?:[\s\n\"<]|$)',  #version 0 looks for something like "arXiv:YYMM.number(vn)" 
                                                                             #where YYMM are 4 digits, numbers are up to 5 digits and (vn) is
                                                                             #an additional optional term specifying the version. n is an integer starting from 1
                                                                             #This is the official format for Arxiv indentifier starting from 1 April 2007
-                '(\d{4}\.\d+)(?:v\d+)?(?:\.pdf)'] 
+                                                                            #This is the official format for Arxiv indentifier starting from 1 April 2007
+                '(\d{4}\.\d+)(?:v\d+)?(?:\.pdf)',                         #version 1 is similar to version 0, without the requirement of "arxiv : " at the beginning 
+                                                                            #but with the requirement that '.pdf' appears right after the possible arXiv identifier. 
+                                                                            #This is helpful when we are trying to extrat the arXiv ID from the file name.
+                '\A(\d{4}\.\d+)(?:v\d+)?$']                               #version 2 is similar to version 0, without the requirement of "arxiv : " at the beginning 
+                                                                            #but requires that the string contains ONLY the arXiv ID.
+    
 
 def validate(identifier,what='doi'):
     """
@@ -58,7 +76,7 @@ def validate(identifier,what='doi'):
     if not identifier:
         return None
     if what=='doi':
-        if re.match(doi_regexp[1],identifier,re.I):
+        if re.match('10\.\d{4,9}/[-._;()/:A-Z0-9]+',identifier,re.I):
             if config.check_online_to_validate:
                 logger.info(f"Validating the possible DOI {identifier} via a query to dx.doi.org...")
                 result = bibtex_makers.doi2bib(identifier)
@@ -77,7 +95,7 @@ def validate(identifier,what='doi'):
         else: return False
 
     elif what=='arxiv':
-        if re.match(r'(\d{4}\.\d+(?:v\d+)?)',identifier,re.I):
+        if re.match(arxiv2007_pattern,identifier,re.I):
             if config.check_online_to_validate:
                 logger.info(f"Validating the possible arxiv ID {identifier} via a query to export.arxiv.org...")
                 result = bibtex_makers.arxiv2bib(identifier)
@@ -115,11 +133,13 @@ def extract_arxivID_from_text(text,version=0):
     -------
     arxivID : string or None
         It returns the arxiv ID if any was found, or None
-    """                                               
-    regexDOI = re.search(arxiv_regexp[version] ,text,re.I)
-    if regexDOI:
-        print(regexDOI.group(1))
-        return regexDOI.group(1)
+    """   
+    try:                                            
+        regexDOI = re.search(arxiv_regexp[version] ,text,re.I)
+        if regexDOI:
+            return regexDOI.group(1)
+    except:
+        pass
     return None
 
 def extract_doi_from_text(text,version=0):
@@ -141,12 +161,17 @@ def extract_doi_from_text(text,version=0):
         It returns the doi if any was found, or None
 
     """    
-    regexDOI = re.search(doi_regexp[version],text,re.I)
-    if regexDOI:
-        return regexDOI.group(1)
+    try:
+        regexDOI = re.search(doi_regexp[version],text,re.I)
+        if regexDOI:
+            return regexDOI.group(1)
+    except:
+        pass
     return None
 
 def find_identifier_in_google_search(query,func_validate,numb_results=10):
+    headers = {"User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/83.0.4103.97 Safari/537.36"}
+
     MaxLengthDisplay = 100
     if len(query)>MaxLengthDisplay:
         query_to_display = query[0:MaxLengthDisplay]  + " ...[query too long, the remaining part is suppressed in the logging]"
@@ -158,7 +183,7 @@ def find_identifier_in_google_search(query,func_validate,numb_results=10):
     try:
         for url in search(query, stop=numb_results):
             logger.info(f"Looking for a valid identifier in the search result #{str(i)} : {url}")
-            response = requests.get(url)
+            response = requests.get(url,headers=headers)
             text = response.text
             identifier,desc,info = find_identifier_in_text(text,func_validate)
             if identifier: 
@@ -198,6 +223,10 @@ def find_identifier_in_text(texts,func_validate):
     if not isinstance(texts,list): texts = [texts]
     
     for text in texts:
+        #logger.info(text)
+        #logger.info(type(text))
+        if isinstance(text, bytes):
+            text = text.decode()
         #First we look for DOI
         for v in range(len(doi_regexp)):
             identifier = extract_doi_from_text(text,version=v)
@@ -356,7 +385,12 @@ def add_found_identifier_to_metadata(path,identifier):
         writer = PdfFileWriter()
         writer.appendPagesFromReader(pdf)
         metadata = pdf.getDocumentInfo()
-        writer.addMetadata(metadata)
+        try:
+            writer.addMetadata(metadata)    #This instruction might generate an error if the pre-existing metadata are weird and are not
+                                            #correctly seen as strings (happens with old files). Therefore we use the try/except
+                                            #to ignore this possible problem
+        except:
+            pass
         key = '/identifier'
         writer.addMetadata({
             key: identifier
@@ -371,7 +405,7 @@ def add_found_identifier_to_metadata(path,identifier):
         return True
     except Exception as e:
         logger.error("Error from PyPDF2: " + str(e))
-        logger.error("An error occured while trying to write the identifier \'{identifier}\' into the metadata of the file \'{path}\'...")
+        logger.error(f"An error occured while trying to write the identifier \'{identifier}\' into the metadata of the file \'{path}\'...")
         return False
 
 
@@ -441,7 +475,6 @@ def find_identifier(path,method,func_validate=validate,**kwargs):
     return result
 
 def find_identifier_by_googling_title(path, func_validate, numb_results=config.numb_results_google_search):
-    logger.info("Looking for possible publication titles...")
     titles = find_possible_titles(path)
 
     if titles:
@@ -464,7 +497,6 @@ def find_identifier_by_googling_title(path, func_validate, numb_results=config.n
         return None, None, None
     
 def find_identifier_by_googling_first_N_characters_in_pdf(path, func_validate, numb_results=config.numb_results_google_search, numb_characters=config.N_characters_in_pdf):
-    logger.info(f"Trying to do a google search with the first {numb_characters} characters of this pdf file...")
     
     if config.websearch==False:
         logger.warning("Web-search methods are currently disabled by the user. Enable it in order to use this method.")
@@ -517,8 +549,7 @@ def find_identifier_in_pdf_info(path,func_validate,keysToCheckFirst=[]):
     -------
     result : dictionary with identifier and other info (see above) 
     """
-    
-    logger.info("Looking for a valid identifier in the document infos...")
+
     pdfinfo = get_pdf_info(path)
     identifier, desc, info = None, None, None
     if pdfinfo:
@@ -548,7 +579,6 @@ def find_identifier_in_filename(path, func_validate):
     -------
     result : dictionary with identifier and other info (see above)
     """
-    logger.info("Looking for a valid identifier in the file name...")
     text = os.path.basename(path)
     identifier,desc,info = find_identifier_in_text([text],func_validate)
     if identifier: 
@@ -571,20 +601,20 @@ def find_identifier_in_pdf_text(path, func_validate):
     -------
     result : dictionary with identifier and other info (see above)
     """
-    logger.info("Looking for a valid identifier in the document text...")
     for reader in reader_libraries:
         logger.info(f"Extracting text with the library {reader}...")
         texts = get_pdf_text(path,reader.lower())
         
         if not isinstance(texts,list):
             texts = [texts]
-        logger.info(f"Text extracted succesfully. Looking for an identifier in the text...")
-        identifier,desc,info = find_identifier_in_text(texts,func_validate)
-        if identifier: 
-            logger.info(f"A valid {desc} was found in the document text.")
-            return identifier,desc,info
-        else:
-            logger.info(f"Could not find a valid identifier in the document text extracted by {reader}.")
+        if texts:
+            logger.info(f"Text extracted succesfully. Looking for an identifier in the text...")
+            identifier,desc,info = find_identifier_in_text(texts,func_validate)
+            if identifier: 
+                logger.info(f"A valid {desc} was found in the document text.")
+                return identifier,desc,info
+            else:
+                logger.info(f"Could not find a valid identifier in the document text extracted by {reader}.")
     logger.info("Could not find a valid identifier in the document text.")
     return None, None, None
         
